/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
/*
 * Copyright (C) 2015 Red Hat
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Written by:
 * 	Matthias Clasen <mclasen@redhat.com>
 */

/* License page {{{1 */

#define PAGE_ID "license"

#include "config.h"
#include "license-resources.h"
#include "gis-license-page.h"

#ifdef HAVE_WEBKITGTK
#include <webkit/webkit.h>
#endif

#include <locale.h>
#include <gtk/gtk.h>

#include "gis-page-header.h"

struct _GisLicensePagePrivate
{
  GtkTextBuffer *lic_textbuffer;
  GSettings *license_settings;
  guint abrt_watch_id;
};
typedef struct _GisLicensePagePrivate GisLicensePagePrivate;

G_DEFINE_TYPE_WITH_PRIVATE (GisLicensePage, gis_license_page, GIS_TYPE_PAGE);

#ifdef HAVE_WEBKITGTK
static void
notify_progress_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
{
  GtkWidget *progress_bar = user_data;
  WebKitWebView *web_view = WEBKIT_WEB_VIEW (object);
  gdouble progress;

  progress = webkit_web_view_get_estimated_load_progress (web_view);

  gtk_widget_set_visible (progress_bar, progress != 1.0);
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), progress);
}

static gboolean
activate_link (GtkLabel       *label,
               const gchar    *uri,
               GisLicensePage *page)
{
  GtkWidget *headerbar;
  GtkWidget *dialog;
  GtkWidget *overlay;
  GtkWidget *view;
  GtkWidget *progress_bar;

  headerbar = gtk_header_bar_new ();
  gtk_header_bar_set_show_title_buttons (GTK_HEADER_BAR (headerbar), TRUE);

  dialog = g_object_new (GTK_TYPE_WINDOW,
                         "destroy-with-parent", TRUE,
                         "transient-for", gtk_widget_get_root (GTK_WIDGET (page)),
                         "titlebar", headerbar,
                         "title", _("License Policy"),
                         "modal", TRUE,
                         NULL);

  overlay = gtk_overlay_new ();
  gtk_window_set_child (GTK_WINDOW (dialog), overlay);

  progress_bar = gtk_progress_bar_new ();
  gtk_widget_add_css_class (progress_bar, "osd");
  gtk_widget_set_halign (progress_bar, GTK_ALIGN_FILL);
  gtk_widget_set_valign (progress_bar, GTK_ALIGN_START);
  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), progress_bar);

  view = webkit_web_view_new ();
  gtk_widget_set_size_request (view, 600, 500);
  gtk_widget_set_hexpand (view, TRUE);
  gtk_widget_set_vexpand (view, TRUE);
  g_signal_connect (view, "notify::estimated-load-progress",
                    G_CALLBACK (notify_progress_cb), progress_bar);
  gtk_overlay_set_child (GTK_OVERLAY (overlay), view);

  gtk_window_present (GTK_WINDOW (dialog));

  webkit_web_view_load_uri (WEBKIT_WEB_VIEW (view), uri);

  return TRUE;
}
#endif

static gboolean
update_os_data (GisLicensePage *page)
{
  GisLicensePagePrivate *priv = gis_license_page_get_instance_private (page);
  g_autofree char *name = g_get_os_info (G_OS_INFO_KEY_NAME);
  g_autofree char *subtitle = NULL;

  if (!name)
    return FALSE;


  return TRUE;
}

static void
abrt_appeared_cb (GDBusConnection *connection,
                  const gchar     *name,
                  const gchar     *name_owner,
                  gpointer         user_data)
{
  GisLicensePage *page = user_data;
  GisLicensePagePrivate *priv = gis_license_page_get_instance_private (page);
}

static void
abrt_vanished_cb (GDBusConnection *connection,
                  const gchar     *name,
                  gpointer         user_data)
{
  GisLicensePage *page = user_data;
  GisLicensePagePrivate *priv = gis_license_page_get_instance_private (page);
}

static void
gis_license_page_constructed (GObject *object)
{
  GisLicensePage *page = GIS_LICENSE_PAGE (object);
  GisLicensePagePrivate *priv = gis_license_page_get_instance_private (page);
  g_autofree gchar *contents = NULL;

  G_OBJECT_CLASS (gis_license_page_parent_class)->constructed (object);

  //gis_page_set_complete (GIS_PAGE (page), TRUE);

  g_assert_true (g_file_get_contents ("/usr/share/sphere-release/EULA", &contents, NULL, NULL));
  int len = strlen(contents);
  gtk_text_buffer_set_text(priv->lic_textbuffer, contents, len);

  if (update_os_data (page))
    {
      priv->abrt_watch_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
                                              "org.freedesktop.problems.daemon",
                                              G_BUS_NAME_WATCHER_FLAGS_NONE,
                                              abrt_appeared_cb,
                                              abrt_vanished_cb,
                                              page,
                                              NULL);
    }
}

static void
accept_button_clicked (GisLicensePage *page)
{
  gis_page_set_complete (GIS_PAGE (page), TRUE);
}

static void
gis_license_page_dispose (GObject *object)
{
  GisLicensePage *page = GIS_LICENSE_PAGE (object);
  GisLicensePagePrivate *priv = gis_license_page_get_instance_private (page);

  g_clear_handle_id (&priv->abrt_watch_id, g_bus_unwatch_name);

  G_OBJECT_CLASS (gis_license_page_parent_class)->dispose (object);
}

static gboolean
gis_license_page_apply (GisPage *gis_page,
                        GCancellable *cancellable)
{
  GisLicensePage *page = GIS_LICENSE_PAGE (gis_page);
  GisLicensePagePrivate *priv = gis_license_page_get_instance_private (page);
  gboolean active;

  //active = gtk_widget_is_visible (priv->reporting_switch) && gtk_switch_get_active (GTK_SWITCH (priv->reporting_switch));
  g_settings_set_boolean (priv->license_settings, "report-technical-problems", active);

  return FALSE;
}

static void
gis_license_page_locale_changed (GisPage *page)
{
  gis_page_set_title (GIS_PAGE (page), _("License"));
}

static void
gis_license_page_class_init (GisLicensePageClass *klass)
{
  GisPageClass *page_class = GIS_PAGE_CLASS (klass);
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

  gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/initial-setup/gis-license-page.ui");
  gtk_widget_class_bind_template_callback (widget_class, accept_button_clicked);

  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisLicensePage, lic_textbuffer);

  page_class->page_id = PAGE_ID;
  page_class->locale_changed = gis_license_page_locale_changed;
  page_class->apply = gis_license_page_apply;
  object_class->constructed = gis_license_page_constructed;
  object_class->dispose = gis_license_page_dispose;
}

static void
gis_license_page_init (GisLicensePage *page)
{
  g_type_ensure (GIS_TYPE_PAGE_HEADER);
  gtk_widget_init_template (GTK_WIDGET (page));
}

GisPage *
gis_prepare_license_page (GisDriver *driver)
{
  return g_object_new (GIS_TYPE_LICENSE_PAGE,
                       "driver", driver,
                       NULL);
}
